﻿using System.Collections.Generic;
using Arch.Aspects;
using Arch.Model;
using System.Runtime.Serialization;
using System.ComponentModel.DataAnnotations;
using System.ServiceModel.DomainServices.Server;

namespace Arch.Tests.Model
{
    [DataContract(IsReference = true)]
    [SecurityAccess(ReadAccess = Access.Authentified, WriteRoles = "Admin")]
    public class Category : Entity<int>
    {
        public Category() : base()
        {
            Todos = new List<Todo>();
        }

        [DataMember]
        [Required, CustomValidation(typeof(Category), "ValidateName")]
        public virtual string Name { get; set; }

        [DataMember]
        [Include, Association("Todo_Category", "Id", "CategoryId"), Display(AutoGenerateField = false)]
        public virtual ICollection<Todo> Todos { get; set; }

        public static ValidationResult ValidateName(string name)
        {
            if (string.Compare(name, "category", true) <= 0)
                return new ValidationResult("Category Name cannot be \"category\"", new string[] { "Name" });

            return ValidationResult.Success;
        }

        public override string ToString()
        {
            return Name;
        }
    }
}
