﻿using System;
using Arch.Model;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;
using Arch.Aspects;
using System.ServiceModel.DomainServices.Server;

namespace Arch.Tests.Model
{
    [DataContract(IsReference = true)]
    [Order("Title"), IncludesAttribute("Category")]
    [SecurityAccess(ReadAccess = Access.Authentified, WriteRoles = "Admin, User")]
    public class Todo : Entity<int>
    {
        [DataMember]
        [Required, StringLength(50)]
        [Display(Order = 0)]
        public virtual string Title { get; set; }

        [DataMember]
        [Required, StringLength(1000), Display(AutoGenerateField = false)]
        public virtual string Description { get; set; }

        [DataMember]
        [Increment]
        [Timestamp]
        public virtual int Version { get; set; }

        [DataMember]
        [AutoNowCreate]
        [Editable(false), DisplayFormat(DataFormatString = "d")]
        public virtual DateTime CreationDate { get; set; }

        [DataMember]
        [AutoNow]
        [Editable(false), DisplayFormat(DataFormatString = "d")]
        public virtual DateTime UpdateDate { get; set; }

        [DataMember]
        [Include, Association("Todo_Category", "CategoryId", "Id", IsForeignKey = true)]
        public virtual Category Category { get; set; }

        [DataMember]
        [Display(AutoGenerateField = false)]
        public virtual int CategoryId { get; set; }

        public override string ToString()
        {
            return Title;
        }
    }
}
