﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Diagnostics;
using System.Reflection;
using Arch.Common.Reflection;

namespace Arch.Tests.Reflection
{
    class EntityTest
    {
        public int Count { get; set; }

        public void Add()
        {
            Count++;
        }
    }

    [TestClass]
    public class MethodInfoExtentionsTest
    {
        [TestMethod]
        public void TestCompile()
        {
             var testcount = new int[] { 10000000 };
            EntityTest test;
            DateTime start, end;
            MethodInfo methodInfo;
            var emptyTab = new object[0];


            foreach (var count in testcount)
            {
                //Normal
                start = DateTime.Now;
                test = new EntityTest();
                for (int i = 0; i < count; i++)
                {
                    test.Add();
                }
                end = DateTime.Now;
                Trace.WriteLine(string.Format("{0} normal calls in {1}", count, end - start));


                //Reflection call
                start = DateTime.Now;
                test = new EntityTest();
                methodInfo = typeof(EntityTest).GetMethod("Add");

                for (int i = 0; i < count; i++)
                {
                    methodInfo.Invoke(test, emptyTab);
                }
                end = DateTime.Now;
                Trace.WriteLine(string.Format("{0} reflection calls in {1}", count, end - start));

                //Expression call
                start = DateTime.Now;
                test = new EntityTest();
                methodInfo = typeof(EntityTest).GetMethod("Add");
                var action = methodInfo.Compile();

                for (int i = 0; i < count; i++)
                {
                    action(test);
                }
                end = DateTime.Now;
                Trace.WriteLine(string.Format("{0} expression calls in {1}", count, end - start));
            }
        }
    }
}
